// ------------------------------------------------------------------------
// Convex Hull -- Tecgraf/PUC-Rio/IMPA
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2008/09/06 19:21:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _CONVEX_HULL_H_
#define _CONVEX_HULL_H_

#include <vector>
#include "coord.h"

using std::vector;

class cConvexHull
{
private:
 // domain points
 vector<cCoord> points;

 // domain sorted points (scanning algorithm)
 vector<cCoord> sorted_points;

 // solution points
 vector<cCoord> result;

 int num_points;
 int num_points_result;

 bool has_result;
 bool has_elimination;
 
 // min / max interior points elimination
 cCoord minX;
 cCoord minY;
 cCoord maxX;
 cCoord maxY;

 //char format[5];

 // check the next point of jarvis
 bool mostUpRight(const cCoord& u, const cCoord& v);

 // merge sort algorithm
 void merge_sort(void);
 void sort(vector<cCoord>* v);
 void merge(vector<cCoord>* v1, vector<cCoord>* v2, vector<cCoord>* v);

public:
 cConvexHull(vector<cCoord>& pts) { reset(pts); };
 ~cConvexHull();

 void reset(vector<cCoord>& pos);

 bool hasResult(void) { return has_result; }
 void hasResult(bool b) { has_result = b; }

 bool hasElimination(void) { return has_elimination; }
 void hasElimination(bool b) { has_elimination = b; }

 // lexographical order
 cCoord lexoMinX(cCoord p1, cCoord p2);
 cCoord lexoMaxX(cCoord p1, cCoord p2);
 cCoord lexoMinY(cCoord p1, cCoord p2);
 cCoord lexoMaxY(cCoord p1, cCoord p2);

 // Interior point elimination
 void interior_point_elimination(void);

 // Jarvis Algorithm (1972)
 void jarvis(void);

 // Scanning Algorithm
 void scanning(void);

 void draw(void);
 void draw_elimination(void);
};

#endif
