// ------------------------------------------------------------------------
// Particle System -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/10/10 19:21:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _COORD_H_
#define _COORD_H_

#include <math.h>
#include "util.h"

class cCoord
{
public:
 unsigned int id;
 double x, y, z;

 cCoord() : id(0), x(0.0), y(0.0), z(0.0) {}
 cCoord(double V) : x(V), y(V), z(V) {}
 cCoord(unsigned int i, double V) : id(i), x(V), y(V), z(V) {}
 cCoord(double X, double Y, double Z) : x(X), y(Y), z(Z) {}
 cCoord(unsigned int i, double X, double Y, double Z) : id(i), x(X), y(Y), z(Z) {}
 cCoord(const cCoord &coord) { *this = coord; }

 void operator = (const cCoord &coord)
 { id = coord.id; x = coord.x; y = coord.y; z = coord.z; }
 void operator = (const double c)
 { x = c; y = c; z = c; }
 void operator += (const cCoord &coord)
 { x += coord.x; y += coord.y; z += coord.z; }
 void operator -= (const cCoord &coord)
 { x -= coord.x; y -= coord.y; z -= coord.z; }
 void operator *= (const cCoord &coord)
 { x *= coord.x; y *= coord.y; z *= coord.z; }
 void operator /= (const cCoord &coord)
 { x /= coord.x; y /= coord.y; z /= coord.z; }
 double modulo(void) const
 { return sqrt((x*x)+(y*y)+(z*z)); }
 void unit(void)
 { x /= modulo(); y /= modulo(); z /= modulo(); }
 
 friend cCoord clamp(cCoord v, cCoord min, cCoord max)
 {
  cCoord res;
  res.x = (v.x > min.x) ? v.x : min.x;
  res.y = (v.y > min.y) ? v.y : min.y;
  res.z = (v.z > min.z) ? v.z : min.z;
  
  res.x = (res.x < max.x) ? res.x : max.x;
  res.y = (res.y < max.y) ? res.y : max.y;
  res.z = (res.z < max.z) ? res.z : max.z;
  
  return res;
 }

 friend cCoord floor(cCoord v)
 {
  cCoord res;

  res.x = (double)((int)v.x);
  res.y = (double)((int)v.y);
  res.z = (double)((int)v.z);

  return res;
 }

 friend bool operator != (const cCoord a, const cCoord b)
 { return ((a.x != b.x) || (a.y != b.y) || (a.z != b.z)) ? true : false; }
 friend cCoord operator + (const cCoord &p1, const cCoord &p2)
 { cCoord p; p.x = p1.x + p2.x; p.y = p1.y + p2.y; p.z = p1.z + p2.z; return p; }
 friend cCoord operator + (const cCoord &p1, double c)
 { cCoord p; p.x = p1.x + c; p.y = p1.y + c; p.z = p1.z + c; return p; }
 friend cCoord operator - (const cCoord &p1, const cCoord &p2)
 { cCoord p; p.x = p1.x - p2.x; p.y = p1.y - p2.y; p.z = p1.z - p2.z; return p; }
 friend cCoord operator - (const cCoord &p1, double c)
 { cCoord p; p.x = p1.x - c; p.y = p1.y - c; p.z = p1.z - c; return p; }
 friend cCoord operator * (const cCoord &p1, double c)
 { cCoord p; p.x = p1.x * c; p.y = p1.y * c; p.z = p1.z * c; return p; }
 friend cCoord operator * (const cCoord &p1, const cCoord &p2)
 { cCoord p; p.x = p1.x * p2.x; p.y = p1.y * p2.y; p.z = p1.z * p2.z; return p; }
 friend cCoord operator / (const cCoord &p1, double c)
 { cCoord p; p.x = p1.x / c; p.y = p1.y / c; p.z = p1.z / c; return p; }
 friend cCoord operator / (double c, const cCoord &p1)
 { cCoord p; p.x = c / p1.x; p.y = c / p1.y; p.z = c / p1.z; return p; }

};

#endif
