// ------------------------------------------------------------------------
// Daluany-- Tecgraf/PUC-Rio/IMPA
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2008/11/15 18:09:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifdef WIN32
#include <windows.h>
#endif

#include <gl/gl.h>
#include <gl/glu.h>
#include <float.h>
#include <algorithm>
#include <vr/timer.h>

#include "delaunay.h"
#include "util.h"
#include "edge.h"
#include "cell.h"

extern void printCell(Cell *cell);

extern void displayScreenMessage( const char* s, float x, float y );

static vr::Timer timer;

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::reset(std::vector<cCoord>& pts)
{
 points.clear();
 num_points = (int)pts.size();
 points.resize(num_points);
 has_result = false;
 has_delaunay = false;
 has_voronoi = false;

 for(int i = 0; i < num_points; i++)
  points[i] = pts[i];

 if(cell)
  Cell::kill(cell);

 cell = new Cell();
 face = Face::make(cell);
}


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::merge_sort(void)
{
 if(num_points < 2)
  return;

 sort(&points);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::sort(vector<cCoord>* v)
{
 int size = (int)v->size();

 // if 1 element need no sort
 if(size < 2)
  return;

 int i, j, half = size / 2;
 vector<cCoord> v1(half), v2(size - half);

 // dividing into two halves
 for(i = 0; i < half; i++)
  v1[i] = (*v)[i];

 for(j = 0, i = half; i < size; j++, i++)
  v2[j] = (*v)[i];

 // sorting the halves 
 sort(&v1);
 sort(&v2);

 // merging the result
 merge(&v1, &v2, v);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::merge(vector<cCoord>* v1, vector<cCoord>* v2, vector<cCoord>* res)
{
 int size1 = v1->size();
 int size2 = v2->size();

 int i = 0, it1 = 0, it2 = 0;

 // stop when the two vecs finish
 while(it1 < size1 || it2 < size2)
 {
  if(it1 >= size1)
  {
   (*res)[i++] = (*v2)[it2++];
   continue;
  }
  else if(it2 >= size2)
  {
   (*res)[i++] = (*v1)[it1++];
   continue;
  }

  // if x is equal, check y
  if(fabs((*v1)[it1].x - (*v2)[it2].x) < TOL_ZERO)
  {
   // if y is equal too, put anyone and increment pointer
   if(fabs((*v1)[it1].y - (*v2)[it2].y) < TOL_ZERO)
    (*res)[i++] = (*v1)[it1++];
   // put the less one and increment pointer
   else if((*v1)[it1].y < (*v2)[it2].y)
    (*res)[i++] = (*v1)[it1++];
   else
    (*res)[i++] = (*v2)[it2++];
  }
  // put the less one and increment pointer
  else
   if((*v1)[it1].x < (*v2)[it2].x)
    (*res)[i++] = (*v1)[it1++];
   else
    (*res)[i++] = (*v2)[it2++];
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::delau(vector<cCoord>* pts)
{
 Vertex *vertex1 = Vertex::make(cell);
 Vertex *vertex2 = Vertex::make(cell);
 Face   *face   = Face::make(cell);
 Edge   *e1   = Edge::make();

 Vertex* vertex3 = Vertex::make(cell);
 Edge* e2 = Edge::make();

 Edge::splice(e1->Sym(), e2);

 e1->setOrg(vertex1);
 e1->setDest(vertex2);
 e1->setLeft(face);
 e1->setRight(face);

 Edge *edge1 = vertex1->getEdge();
 Face *left  = edge1->Left();
 Face *right = edge1->Right();

 e2->setOrg(vertex2);
 e2->setDest(vertex3);
 e2->setLeft(left);
 e2->setRight(right);

 Edge* e3 = Connect(e2, e1, right);
 MakeLeftFace(e2, e1, e3);


 //cell->makeFaceEdge(left, vertex3, vertex1);

 // drop in four vertices along the initial edge

 //Vertex *vertex2 = cell->makeVertexEdge(vertex1, left, right)->Dest();
 //
 //
 //Vertex *vertex3 = Vertex::make(cell);
 //Face   *l3   = Face::make(cell);
 ////Face   *r3  = Face::make(cell);
 //Edge   *e3   = Edge::make()->InvRot();

 //e3->setOrg(vertex3);
 //e3->setDest(vertex3);
 //e3->setLeft(l3);
 //e3->setRight(right);

 //edge1 = vertex3->getEdge();
 //left  = edge1->Left();
 //right = edge1->Right();

 //// drop in four vertices along the initial edge

 ////Vertex *vertex4 = cell->makeVertexEdge(vertex3, left, right)->Dest();
 ////Vertex *vertex4 = cell->makeVertexEdge(vertex3, left, right)->Dest();

 //Edge* d = Connect(vertex2->getEdge(), vertex3->getEdge(), right);
 //Edge* c = Connect(vertex3->getEdge(), vertex2->getEdge(), right);

 //// cut each of the faces in half from complementary vertices

 ////Face *front  = cell->makeFaceEdge(left, vertex2, vertex4)->Right();
 ////Face *bottom = cell->makeFaceEdge(right, vertex3, vertex2)->Right();
 //cell->killFaceEdge(
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::SetVoronoi(Cell* cell)
{
 voronoi.clear();
 voronoi.resize(cell->countFaces()-1);

 double s;
 cCoord face_v;
 cCoord v, q;
 cCoord M, N;
 cCoord pA, pB, pC, pD;

 cCoord* p;

 CellFaceIterator faces(cell);
 Face *face;
 Edge *e1, *e2;

 int i = 0;
 while((face = faces.next())!=0 && face->getID() != 1)
 {
  FaceEdgeIterator edges(face);

  e1 = edges.next();
  e2 = e1->Lnext();

  pA = e1->Org()->pos;
  pB = e1->Dest()->pos;
  pC = e2->Org()->pos;
  pD = e2->Dest()->pos;

  v.x = pA.y - pB.y;
  v.y = pB.x - pA.x;

  q.x = pC.y - pD.y;
  q.y = pD.x - pC.x;

  M.x = (pA.x + pB.x) / 2.0;
  M.y = (pA.y + pB.y) / 2.0;

  N.x = (pC.x + pD.x) / 2.0;
  N.y = (pC.y + pD.y) / 2.0;

  s = (1.0/((v.x*q.y)-(v.y*q.x)))*((v.y*(N.x-M.x))+(v.x*(M.y-N.y)));

  face_v.x = N.x + (s * q.x);
  face_v.y = N.y + (s * q.y);

  p = new cCoord(face_v);

  face->data = (void*)p;

  voronoi[i++] = face_v;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::SetVoronoi(Face* face)
{
 double s;
 cCoord* p;
 cCoord face_v;
 cCoord v, q;
 cCoord M, N;
 cCoord pA, pB, pC, pD;

 Edge *e1, *e2;
 FaceEdgeIterator edges(face);

 e1 = edges.next();
 e2 = e1->Lnext();

 pA = e1->Org()->pos;
 pB = e1->Dest()->pos;
 pC = e2->Org()->pos;
 pD = e2->Dest()->pos;

 v.x = pA.y - pB.y;
 v.y = pB.x - pA.x;

 q.x = pC.y - pD.y;
 q.y = pD.x - pC.x;

 M.x = (pA.x + pB.x) / 2.0;
 M.y = (pA.y + pB.y) / 2.0;

 N.x = (pC.x + pD.x) / 2.0;
 N.y = (pC.y + pD.y) / 2.0;

 s = (1.0/((v.x*q.y)-(v.y*q.x)))*((v.y*(N.x-M.x))+(v.x*(M.y-N.y)));

 face_v.x = N.x + (s * q.x);
 face_v.y = N.y + (s * q.y);

 p = new cCoord(face_v);

 face->data = (void*)p;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::divide_and_conquer(void)
{
 if(num_points < 2)
  return;

 timer.restart();

 this->merge_sort();

 this->delaunay_alg(&points, &left_edge, &right_edge);

 double time = timer.elapsed();
 printf("Algorithm : %.4e ms  %d\n", time * 1000, (int)points.size());

 has_result = true;
 has_delaunay = true;
 has_voronoi = true;

 if(num_points < 1000)
  printCell(cell);
}


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::delaunay_alg(vector<cCoord>* pts, Edge** l_e, Edge** r_e)
{
 int size = (int)pts->size();
 if(size == 2)
 {
  Vertex *s1 = Vertex::make(cell);
  s1->pos = (*pts)[0];

  Vertex *s2 = Vertex::make(cell);
  s2->pos = (*pts)[1];
  
  Edge *a = Edge::make();

  a->setOrg(s1);
  a->setDest(s2);
  a->setLeft(face);
  a->setRight(face);

  (*l_e) = a;
  (*r_e) = a->Sym();
  return;
 }
 else if(size == 3)
 {
  Vertex* s1 = Vertex::make(cell);
  s1->pos = (*pts)[0];

  Vertex* s2 = Vertex::make(cell);
  s2->pos = (*pts)[1];
  
  Vertex* s3 = Vertex::make(cell);
  s3->pos = (*pts)[2];

  Edge* a = Edge::make();
  Edge* b = Edge::make();

  Edge::splice(a->Sym(), b);

  a->setOrg(s1);
  a->setDest(s2);
  a->setLeft(face);
  a->setRight(face);

  Edge *edge1 = s1->getEdge();
  Face *left  = edge1->Left();
  Face *right = edge1->Right();

  b->setOrg(s2);
  b->setDest(s3);
  b->setLeft(left);
  b->setRight(right);

  if(CCW((*pts)[0], (*pts)[1], (*pts)[2]))
  {
   Edge* c = this->Connect(b, a, face);
   this->MakeLeftFace(b, a, c);

   (*l_e) = a;
   (*r_e) = b->Sym();
   return;
  }
  else if(CCW((*pts)[0], (*pts)[2], (*pts)[1]))
  {
   Edge* c = this->Connect(b, a, face);
   this->MakeLeftFace(b, a, c);

   Face* f = a->Left();

   a->setLeft(face);
   b->setLeft(face);
   c->setLeft(face);

   a->setRight(f);
   b->setRight(f);
   c->setRight(f);

   (*l_e) = c->Sym();
   (*r_e) = c;
   return;
  }
  else
  {
   (*l_e) = a;
   (*r_e) = b->Sym();
   return;
  }
 }
 else
 {
  int i, j, half = size/2;
  vector<cCoord> L(half), R(size - half);

  // dividing into two halves
  for(i = 0; i < half; i++)
   L[i] = (*pts)[i];

  for(j = 0, i = half; i < size; j++, i++)
   R[j] = (*pts)[i];

  // LEFT DELAUNAY
  Edge* ldo = NULL;
  Edge* ldi = NULL;
  this->delaunay_alg(&L, &ldo, &ldi);

  // RIGHT DELAUNAY
  Edge* rdo = NULL;
  Edge* rdi = NULL;
  this->delaunay_alg(&R, &rdi, &rdo);

  // finding basel
  while(true)
  {
   if(this->LeftOf(rdi->Org()->pos, ldi))
    ldi = ldi->Lnext();
   else if(this->RightOf(ldi->Org()->pos, rdi))
    rdi = rdi->Rprev();
   else
    break;
  }

  // BASEL
  Edge* basel = this->Connect(rdi->Sym(), ldi, face);

  // correcting ldo with the new basel
  if(ldi->Org()->getID() == ldo->Org()->getID())
   ldo = basel->Sym();

  // correcting rdo with the new basel
  if(rdi->Org()->getID() == rdo->Org()->getID())
   rdo = basel;

  // MERGE
  Edge* temp = NULL;
  Edge* lcand = NULL;
  Edge* rcand = NULL;
  while(true)
  {
   // lcand
   lcand = basel->Sym()->Onext();
   bool valid_lcand = this->RightOf(lcand->Dest()->pos, basel);
   if(valid_lcand)
   {
    while(InCircle(basel->Dest()->pos, basel->Org()->pos, lcand->Dest()->pos, lcand->Onext()->Dest()->pos))
    {
     temp = lcand->Onext();
     DeleteEdge(lcand);
     lcand = temp;
    }
   }

   // rcand
   rcand = basel->Oprev();
   bool valid_rcand = this->RightOf(rcand->Dest()->pos, basel);
   if(valid_rcand)
   {
    while(InCircle(basel->Dest()->pos, basel->Org()->pos, rcand->Dest()->pos, rcand->Oprev()->Dest()->pos))
    {
     temp = rcand->Oprev();
     DeleteEdge(rcand);
     rcand = temp;
    }
   }

   // lcand and rcand invalids
   if(!valid_lcand && !valid_rcand)
    break;

   if(!valid_lcand || (valid_rcand && InCircle(lcand->Dest()->pos, lcand->Org()->pos, rcand->Org()->pos, rcand->Dest()->pos)))
   {
    Edge* t = basel->Sym();
    basel = this->Connect(rcand, t, face);
    this->MakeLeftFace(rcand, t, basel);
   }
   else
   {
    Edge* t = basel->Sym();
    basel = this->Connect(t, lcand->Sym(), face);
    this->MakeLeftFace(t, lcand->Sym(), basel);
   }
  }

  (*l_e) = ldo;
  (*r_e) = rdo;
  return;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::MakeLeftFace(Edge* a, Edge* b, Edge* c)
{
 Face* l = Face::make(cell);
 a->setLeft(l);
 b->setLeft(l);
 c->setLeft(l);

 SetVoronoi(l);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
Edge* cDelaunay::Connect(Edge* a, Edge* b, Face* face)
{
 Edge* c = Edge::make();
 c->setOrg(a->Dest());
 c->setDest(b->Org());
 
 Edge::splice(c, a->Lnext());
 Edge::splice(c->Sym(), b);

 c->setLeft(face);
 c->setRight(face);
 
 return c;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::DeleteEdge(Edge* e)
{
 Edge *t1, *t2;

 if(e->Right()->getID() == face->getID())
 {
  t1 = e->Lnext();
  t2 = t1->Lnext();

  t1->setLeft(e->Right());
  t2->setLeft(e->Right());
  Face::kill(e->Left());
 }
 else
 {
  t1 = e->Rprev();
  t2 = t1->Rprev();

  t1->setRight(e->Left());
  t2->setRight(e->Left());
  Face::kill(e->Right());
 }

 Edge::splice(e, e->Oprev());
 Edge::splice(e->Sym(), e->Sym()->Oprev());

 Edge::kill(e);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool cDelaunay::LeftOf(cCoord& x, Edge* e)
{
 return this->CCW(e->Org()->pos, e->Dest()->pos, x);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool cDelaunay::RightOf(cCoord& x, Edge* e)
{
 return this->CCW(e->Dest()->pos, e->Org()->pos, x);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool cDelaunay::CCW(cCoord& p1, cCoord& p2, cCoord& p3)
{
 p1.z = 1.0;
 p2.z = 1.0;
 p3.z = 1.0;
 double det = detMatrix(p1, p2, p3);

 if(det > TOL_ZERO)
  return true;
 else if(det < TOL_ZERO)
  return false;

 return false;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool cDelaunay::InCircle(cCoord& a, cCoord& b, cCoord& c, cCoord& d)
{
 a.z = (a.x * a.x) + (a.y * a.y);
 b.z = (b.x * b.x) + (b.y * b.y);
 c.z = (c.x * c.x) + (c.y * c.y);
 d.z = (d.x * d.x) + (d.y * d.y);
 double res = ((-detMatrix(b, c, d))+(detMatrix(a, c, d))+(-detMatrix(a, b, d)+(detMatrix(a, b, c))));
 return res > TOL_ZERO ? true : false;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDelaunay::draw(void)
{
 if(!has_result)
  return;

 glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LINE_BIT | GL_POINT_BIT | GL_COLOR_BUFFER_BIT);

 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

 glEnable(GL_POINT_SMOOTH);
 glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

 //glLineWidth(2.0f);
 glPointSize(5.0);

 // DELAUNAY TRIANGULATION
 CellFaceIterator faces(cell);
 Face *left, *right;
 Edge* edge;
 cCoord *pLeft, *pRight;

 while((left = faces.next())!=0 && left->getID() != 1)
 {
  FaceEdgeIterator edges(left);
  
  if(!(edge = edges.next()))
   break;

  Vertex* org = edge->Org();

  do
  {
   glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

   if(has_delaunay)
   {
    // delaunay edge
    glBegin(GL_LINES);
    glVertex2f(edge->Org()->pos.x, edge->Org()->pos.y);
    glVertex2f(edge->Dest()->pos.x, edge->Dest()->pos.y);
    glEnd();

    // delaunay vertice
    glBegin(GL_POINTS);
    glVertex2f(edge->Org()->pos.x, edge->Org()->pos.y);
    glVertex2f(edge->Dest()->pos.x, edge->Dest()->pos.y);
    glEnd();
   }

   glColor4f(1.0f, 0.0f, 0.0f, 1.0f);

   if(has_voronoi)
   {
    right = edge->Right();
    if(right->getID() != 1)
    {
     pLeft = (cCoord*)left->data;
     pRight = (cCoord*)right->data;

     // voronoi edge
     glBegin(GL_LINES);
     glVertex2f(pRight->x, pRight->y);
     glVertex2f(pLeft->x, pLeft->y);
     glEnd();

     // voronoi vertices
     glBegin(GL_POINTS);
     glVertex2f(pRight->x, pRight->y);
     glVertex2f(pLeft->x, pLeft->y);
     glEnd();
    }
    else
    {
     cCoord M, n, p;

     pLeft = (cCoord*)left->data;
     M = (edge->Org()->pos + edge->Dest()->pos) / 2.0;

     if(CCW(edge->Org()->pos, edge->Dest()->pos, *pLeft))
      n = M - *pLeft;
     else
      n = *pLeft - M;

     p.x = pLeft->x + (10.0*(1.0 / n.modulo()) * n.x);
     p.y = pLeft->y + (10.0*(1.0 / n.modulo()) * n.y);

     // voronoi edge
     glBegin(GL_LINES);
     glVertex2f(pLeft->x, pLeft->y);
     glVertex2f(p.x, p.y);
     glEnd();
    }
   }

   edge = edges.next();
  }while(edge !=0 && edge->Org()->getID() != org->getID());
 }

 glPopAttrib();
}
