// ------------------------------------------------------------------------
// Delaunay -- Tecgraf/PUC-Rio/IMPA
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2008/11/15 16:42:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _DELAUNAY_H_
#define _DELAUNAY_H_

#include <vector>
#include "coord.h"

using std::vector;

class Edge;
class Cell;
class Vertex;
class Face;

class cDelaunay
{
private:
 // domain points
 vector<cCoord> points;

 vector<cCoord> voronoi;

 // manifold triangulation
 Cell* cell;

 // left and right edges
 Edge* left_edge;
 Edge* right_edge;

 Face* face;

 int num_points;

 bool has_result;
 bool has_delaunay;
 bool has_voronoi;
 
 // merge sort algorithm
 void merge_sort(void);
 void sort(vector<cCoord>* v);
 void merge(vector<cCoord>* v1, vector<cCoord>* v2, vector<cCoord>* v);

 // delaunay
 void delaunay_alg(vector<cCoord>* pts, Edge** l_e, Edge** r_e);

 // create an edge connect from a.dest->b.org
 Edge* Connect(Edge* a, Edge* b, Face* face);

 // true if x is at left of e
 bool LeftOf(cCoord& x, Edge* e);

 // true if x is at right of e
 bool RightOf(cCoord& x, Edge* e);

 // true if p3 is CounterClockWise (CCW) from p1->p2
 bool CCW(cCoord& p1, cCoord& p2, cCoord& p3);

 // true if point d is in circle defined by a,b,c
 bool InCircle(cCoord& a, cCoord& b, cCoord& c, cCoord& d);

 // splice operations to eliminate the edge e
 void DeleteEdge(Edge* e);

 // teste
 void delau(vector<cCoord>* pts);

 void MakeLeftFace(Edge* a, Edge* b, Edge* c);

 // calculate the voronoi vertices
 void SetVoronoi(Cell* cell);

 void SetVoronoi(Face* face);

public:
 cDelaunay(vector<cCoord>& pts) : cell(NULL) { reset(pts); };
 ~cDelaunay();

 void reset(vector<cCoord>& pos);

 bool hasResult(void) { return has_result; }
 void hasResult(bool b) { has_result = b; }

 bool hasDelaunay(void) { return has_delaunay; }
 void hasDelaunay(bool b) { has_delaunay = b; }

 bool hasVoronoi(void) { return has_voronoi; }
 void hasVoronoi(bool b) { has_voronoi = b; }

 void divide_and_conquer(void);

 void draw(void);
};

#endif
