// ------------------------------------------------------------------------
// Particle System -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2008/09/06 19:21:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifdef WIN32
#include <windows.h>
#endif

#include <vr/random.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <float.h>

#include "domain.h"

extern void displayScreenMessage( const char* s, float x, float y );

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::reset(void)
{
 points.clear();
 num_points = 0;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::addPoint(cCoord& p)
{
 points.push_back(p);
 num_points++;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::square(unsigned int npoints, double min, double max)
{
 num_points = npoints;
 min_value = min;
 max_value = max;

 cCoord p;

 points.clear();
 points.resize(num_points);

 this->set_format("%c");

 p.id = A;
 for(unsigned int i = 0; i < num_points; i++)
 {
  p.x = vr::Random::real(min_value, max_value);
  p.y = vr::Random::real(min_value, max_value);
  points[i] = p;
  p.id++;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::triangle(unsigned int npoints, double min, double max)
{
 num_points = npoints;
 min_value = min;
 max_value = max;

 double minX = DBL_MAX;
 double minY = DBL_MAX;
 
 double maxX = -DBL_MAX;
 double maxY = -DBL_MAX;

 bool r1, r2, r3;
 cCoord p, p1, p2, p3, v;
 cCoord u1, u2, u3, ref1, ref2, ref3;

 points.clear();
 points.resize(num_points);

 p1.x = vr::Random::real(min_value, max_value);
 p1.y = vr::Random::real(min_value, max_value);

 p2.x = vr::Random::real(min_value, max_value);
 p2.y = vr::Random::real(min_value, max_value);

 p3.x = vr::Random::real(min_value, max_value);
 p3.y = vr::Random::real(min_value, max_value);

 if(p1.x < minX)
  minX = p1.x;
 if(p1.x > maxX)
  maxX = p1.x;
 if(p1.y < minY)
  minY = p1.y;
 if(p1.y > maxY)
  maxY = p1.y;

 if(p2.x < minX)
  minX = p2.x;
 if(p2.x > maxX)
  maxX = p2.x;
 if(p2.y < minY)
  minY = p2.y;
 if(p2.y > maxY)
  maxY = p2.y;

 if(p3.x < minX)
  minX = p3.x;
 if(p3.x > maxX)
  maxX = p3.x;
 if(p3.y < minY)
  minY = p3.y;
 if(p3.y > maxY)
  maxY = p3.y;

 ref1 = p1;
 u1 = p2 - p1;
 v = p3 - p1;
 if(!left(u1, v))
 {
  ref1 = p2;
  u1 = p1 - p2;
 }

 ref2 = p2;
 u2 = p3 - p2;
 v = p1 - p2;
 if(!left(u2, v))
 {
  ref2 = p3;
  u2 = p2 - p3;
 }

 ref3 = p3;
 u3 = p1 - p3;
 v = p2 - p3;
 if(!left(u3, v))
 {
  ref3 = p1;
  u3 = p3 - p1;
 }

 this->set_format("%c");

 unsigned int i = 0;
 p.id = A;
 while(i < num_points)
 {
  p.x = vr::Random::real(minX, maxX);
  p.y = vr::Random::real(minY, maxY);

  v = p - ref1;
  r1 = left(u1, v);

  v = p - ref2;
  r2 = left(u2, v);

  v = p - ref3;
  r3 = left(u3, v);

  if(r1 && r2 && r3)
  {
   points[i++] = p;
   p.id++;
  }
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::circle(unsigned int npoints, double min, double max, double radius)
{
 num_points = npoints;
 min_value = min;
 max_value = max;

 cCoord p;

 points.clear();
 points.resize(num_points);
 this->set_format("%c");

 unsigned int i = 0;
 p.id = A;
 while(i < num_points)
 {
  p.x = vr::Random::real(min_value, max_value);
  p.y = vr::Random::real(min_value, max_value);

  if((p.x*p.x) + (p.y*p.y) <= (radius*radius))
  {
   points[i++] = p;
   p.id++;
  }
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::on_circle(unsigned int npoints, double min, double max, double radius)
{
 num_points = npoints;
 min_value = min;
 max_value = max;

 cCoord p;
 double angle;

 points.clear();
 points.resize(num_points);
 this->set_format("%c");

 p.id = A;
 for(unsigned int i = 0; i < num_points; i++)
 {
  angle = vr::Random::real(0.0, 2 * MPI);

  p.x = radius * cos(angle);
  p.y = radius * sin(angle);
  points[i] = p;
  p.id++;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::ring(unsigned int npoints, double min, double max, double min_radius, double max_radius)
{
 num_points = npoints;
 min_value = min;
 max_value = max;

 cCoord p;
 double angle, rd;

 points.clear();
 points.resize(num_points);
 this->set_format("%c");

 p.id = A;
 for(unsigned int i = 0; i < num_points; i++)
 {
  rd = vr::Random::real(min_radius, max_radius);
  angle = vr::Random::real(0.0, 2 * MPI);

  p.x = rd * cos(angle);
  p.y = rd * sin(angle);
  points[i] = p;
  p.id++;
 }
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool cDomain::left(const cCoord& u, const cCoord &v)
{
 double det = detCrossProd(u, v);

 if(det >= 0.0)
  return true;
 else
  return false;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::set_format(const char* s)
{
 sprintf(format, "%s", s);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void cDomain::draw(void)
{
 glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_POINT_BIT);

 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 glEnable(GL_POINT_SMOOTH);
 glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
 
 glPointSize(7.0);
 glColor3f(0.0f, 0.0f, 0.0f); 

 char s[5];
 for(unsigned int i = 0; i < num_points; i++)
 {
  glBegin(GL_POINTS);
  glVertex3d(points[i].x, points[i].y, 0.5);
  glEnd();

  if(num_points < 100)
  {
   sprintf(s, format, points[i].id);
   displayScreenMessage((const char*)s, (float)points[i].x + 0.1f, (float)points[i].y + 0.1f);
  }
 }

 glPopAttrib();
}