// ------------------------------------------------------------------------
// Convex Hull -- Tecgraf/PUC-Rio/IMPA
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2008/09/06 19:21:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#ifndef _DOMAIN_H_
#define _DOMAIN_H_

#include <vector>
#include "coord.h"

using std::vector;

class cDomain
{
private:
 vector<cCoord> points;

 unsigned int num_points;
 
 double min_value;
 double max_value;

 char format[5];

 bool left(const cCoord& u, const cCoord &v);

 bool has_domain;

public:
 cDomain() : has_domain(true) { this->reset(); }
 cDomain(unsigned int npoints, double min, double max) : has_domain(true) { this->square(npoints, min, max); };
 ~cDomain() { points.clear(); };

 void reset(void);

 void addPoint(cCoord& p);

 void set_format(const char* s);

 vector<cCoord>& get_points(void) { return points; }

 bool hasDomain(void) { return has_domain; }
 void hasDomain(bool b) { has_domain = b; }

 void square(unsigned int npoints, double min, double max);
 void circle(unsigned int npoints, double min, double max, double radius);
 void on_circle(unsigned int npoints, double min, double max, double radius);
 void triangle(unsigned int npoints, double min, double max);
 void ring(unsigned int npoints, double min, double max, double min_radius, double max_radius);

 void draw(void);

};

#endif
