// ------------------------------------------------------------------------
// Particle System -- Tecgraf/PUC-Rio
// www.tecgraf.puc-rio.br/~lduarte
//
// $Source:  $
// $Revision: 1.0 $
// $Date: 2007/10/10 19:21:00 $
//
// $Author: lduarte $
// ------------------------------------------------------------------------

#include "util.h"
#include "coord.h"

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
double ScalarProd(const cCoord& u, const cCoord& v)
{
 return (u.x * v.x) + (u.y * v.y) + (u.z * v.z);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cCoord Project(const cCoord& u, const cCoord& v)
{
 double scale = (ScalarProd(u, v)) / (ScalarProd(u, v));
 return cCoord(v.x * scale, v.y * scale, v.z * scale);
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cCoord CrossProd(const cCoord& u, const cCoord& v)
{
 cCoord w;

 w.x = u.y * v.z - u.z * v.y;
 w.y = u.z * v.x - u.x * v.z;
 w.z = u.x * v.y - u.y * v.x;

 return w;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
double detCrossProd(const cCoord& u, const cCoord& v)
{
 cCoord w;

 w.x = u.y * v.z - u.z * v.y;
 w.y = u.z * v.x - u.x * v.z;
 w.z = u.x * v.y - u.y * v.x;

 return w.x + w.y + w.z;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
double detMatrix(cCoord& a, cCoord& b, cCoord& c)
{
 return ((a.x * b.y * c.z) + (a.y * b.z * c.x) + (a.z * b.x * c.y)) - 
        ((a.y * b.x * c.z) + (a.x * b.z * c.y) + (a.z * b.y * c.x));
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
double pseudoAngle(const cCoord& u, const cCoord& v)
{
 double scalar = ScalarProd(u, v);
 double modules = u.modulo() * v.modulo();
 
 return 1.0 - ( scalar / modules );
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cCoord Min(cCoord a, cCoord b)
{
 cCoord res;

 a.x < b.x ? res.x = a.x : res.x = b.x;
 a.y < b.y ? res.y = a.y : res.y = b.y;

 return res;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
cCoord Max(cCoord a, cCoord b)
{
 cCoord res;

 a.x > b.x ? res.x = a.x : res.x = b.x;
 a.y > b.y ? res.y = a.y : res.y = b.y;

 return res;
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
bool is_power_of_two(int x)
{
 if((x == 2)    ||
    (x == 4)    ||
    (x == 8)    ||
    (x == 16)   ||
    (x == 32)   ||
    (x == 64 )  ||
    (x == 128)  ||
    (x == 256)  ||
    (x == 512)  ||
    (x == 1024) ||
    (x == 2048) ||
    (x == 4096) ||
    (x == 8192))
  return true;
 else
  return false;
}
